# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# Ported by NoMaY from GNURX version to CC-RL version

set(CMAKE_MODULE_PATH "${CMAKE_CURRENT_LIST_DIR}/../../cmake/Modules")

set(CMAKE_SYSTEM_NAME Generic)
set(CMAKE_SYSTEM_PROCESSOR rl78) # It seems that this variable is not used actually by CMake.

# default to Release build
if(NOT CMAKE_BUILD_TYPE)
    set(CMAKE_BUILD_TYPE "Debug" CACHE STRING "Choose the type of build, options are: Debug Release." FORCE)
endif()

set(TOOLCHAIN_EXT ".exe")

find_program(COMPILER_ON_PATH "ccrl${TOOLCHAIN_EXT}")

if(COMPILER_ON_PATH)
    # check on the current path
    get_filename_component(RL78_TOOLCHAIN_PATH ${COMPILER_ON_PATH} DIRECTORY)
    string(REPLACE "/" "\\" _RL78_TOOLCHAIN_PATH "${RL78_TOOLCHAIN_PATH}")
    message(STATUS "Using RL78 CC-RL from path = ${_RL78_TOOLCHAIN_PATH}")
    unset(_RL78_TOOLCHAIN_PATH)
    # check device file (the following is in case of CS+ IDE)
    if(DEVICE_FILE AND EXISTS "${RL78_TOOLCHAIN_PATH}/../../../Device/RL78/Devicefile/${DEVICE_FILE}")
        set(DEVICE_FILE_SUPPLIED_BY_IDE "${RL78_TOOLCHAIN_PATH}/../../../Device/RL78/Devicefile/${DEVICE_FILE}")
    endif()
else()
    # not found
    message(NOTICE
        "Error: Unable to find RL78 CC-RL compiler on the PATH = $ENV{Path}"
    )
#    message(FATAL_ERROR "Aborted due to runtime environment error.")
set(RL78_TOOLCHAIN_PATH "C:/Renesas/CS+/CC/CC-RL/V1.11.00/bin")
endif()

find_program(CONVERTER_ON_PATH "renesas_cc_converter${TOOLCHAIN_EXT}")

if(CONVERTER_ON_PATH)
    # check on the current path
    get_filename_component(RL78_EXTERNAL_TOOLCHAIN_PATH ${CONVERTER_ON_PATH} DIRECTORY)
    string(REPLACE "/" "\\" _RL78_EXTERNAL_TOOLCHAIN_PATH "${RL78_EXTERNAL_TOOLCHAIN_PATH}")
    message(STATUS "Using e2 studio's X Converter from path = ${_RL78_EXTERNAL_TOOLCHAIN_PATH}")
    unset(_RL78_EXTERNAL_TOOLCHAIN_PATH)
    # check device file (the following is in case of e2 studio IDE)
    if(DEVICE_FILE AND EXISTS "${RL78_EXTERNAL_TOOLCHAIN_PATH}/../../DebugComp/RL78/RL78/Common/${DEVICE_FILE}")
        set(DEVICE_FILE_SUPPLIED_BY_IDE "${RL78_EXTERNAL_TOOLCHAIN_PATH}/../../DebugComp/RL78/RL78/Common/${DEVICE_FILE}")
    endif()
else()
    # not found
    message(NOTICE
        "Warning: Unable to find e2 studio's X Converter on the PATH = $ENV{Path}\n"
        "Debugging with RL78 GDB is not available when the converter is not executed."
    )
endif()

if(DEVICE_FILE_SUPPLIED_BY_IDE)
    message(STATUS "Using RL78 device file = ${DEVICE_FILE_SUPPLIED_BY_IDE}")
else()
    if(NOT DEVICE_FILE)
        # not defined
        message(NOTICE
            "Error: RL78 device file is not specified."
        )
    else()
        # not found
        message(NOTICE
            "Error: Unable to find RL78 device file = ${DEVICE_FILE}"
            # TODO: show more information to solve the problem
        )
    endif()
endif()

# perform compiler test with the static library
#set(CMAKE_TRY_COMPILE_TARGET_TYPE STATIC_LIBRARY)

set(CMAKE_C_COMPILER    ${RL78_TOOLCHAIN_PATH}/ccrl${TOOLCHAIN_EXT} CACHE STRING "")
set(CMAKE_CXX_COMPILER  ${RL78_TOOLCHAIN_PATH}/ccrl${TOOLCHAIN_EXT} CACHE STRING "")
set(CMAKE_ASM_COMPILER  ${RL78_TOOLCHAIN_PATH}/asrl${TOOLCHAIN_EXT})
set(CMAKE_LINKER        ${RL78_TOOLCHAIN_PATH}/rlink${TOOLCHAIN_EXT})
set(CMAKE_AR            ${RL78_TOOLCHAIN_PATH}/rlink${TOOLCHAIN_EXT})
set(RENESAS_XCONVERTER  ${RL78_EXTERNAL_TOOLCHAIN_PATH}/renesas_cc_converter${TOOLCHAIN_EXT})

set(CMAKE_FIND_ROOT_PATH_MODE_PROGRAM NEVER)
set(CMAKE_FIND_ROOT_PATH_MODE_LIBRARY ONLY)
set(CMAKE_FIND_ROOT_PATH_MODE_INCLUDE ONLY)
set(CMAKE_FIND_ROOT_PATH_MODE_PACKAGE ONLY)

set(COMMON_FLAGS    "${CPU_FLAGS} -dev=${DEVICE_FILE_SUPPLIED_BY_IDE} -goptimize -character_set=utf8")
set(CMAKE_C_FLAGS 	"${COMMON_FLAGS} -refs_without_declaration")
#set(CMAKE_CXX_FLAGS "${COMMON_FLAGS} -refs_without_declaration")
set(CMAKE_ASM_FLAGS "${COMMON_FLAGS}")
set(CMAKE_C_LINK_FLAGS     "")
#set(CMAKE_CXX_LINK_FLAGS   "")
set(SYSLIB_FLAGS "")
foreach (_syslibFile ${SYSLIB_FILES})
    set(SYSLIB_FLAGS "${SYSLIB_FLAGS} -library=\"${RL78_TOOLCHAIN_PATH}/../lib/${_syslibFile}\"")
endforeach()
#message(STATUS "SYSLIB_FLAGS=${SYSLIB_FLAGS}")
set(CMAKE_EXE_LINKER_FLAGS "-device=${DEVICE_FILE_SUPPLIED_BY_IDE} ${SYSLIB_FLAGS} -optimize=speed -entry=_start")
set(RENESAS_XCONVERTER_FLAGS "")

#-D RL78_G23_FPB

set(CMAKE_C_COMPILER_ID RENESAS)
set(CMAKE_C_COMPILER_ID_RUN TRUE)
set(CMAKE_C_COMPILER_ARCHITECTURE_ID RL78)
set(CMAKE_C_COMPILER_FORCED TRUE)
#set(CMAKE_CXX_COMPILER_ID RENESAS)
#set(CMAKE_CXX_COMPILER_ID_RUN TRUE)
#set(CMAKE_CXX_COMPILER_ARCHITECTURE_ID RL78)
#set(CMAKE_CXX_COMPILER_FORCED TRUE)
set(CMAKE_ASM_COMPILER_ID RENESAS)
set(CMAKE_ASM_COMPILER_ID_RUN TRUE)
set(CMAKE_ASM_COMPILER_ARCHITECTURE_ID RL78)
set(CMAKE_ASM_COMPILER_FORCED TRUE)
